/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License", destination); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { SideBarView, ViewSection, ViewSectionConstructor } from '../..';
import { AbstractElement } from '../AbstractElement';
/**
 * Page object representing the view container of a side bar view
 */
export declare class ViewContent extends AbstractElement {
    constructor(view?: SideBarView);
    /**
     * Finds whether a progress bar is active at the top of the view
     * @returns Promise resolving to true/false
     */
    hasProgress(): Promise<boolean>;
    /**
     * Retrieves a collapsible view content section by its title.
     * Generic parameter allows caller to cast returned section to
     * desired type, however it is caller's responsibility to check
     * whether type is compatible with desired type.
     * @param title Title of the section
     * @returns Promise resolving to ViewSection object
     */
    getSection<T extends ViewSection>(title: string): Promise<T>;
    /**
     * Retrieves a collapsible view content section by its title.
     * Type parameter allows caller to use custom section implementation,
     * however it is caller's responsibility to check
     * whether type is compatible with desired type.
     * @param title Title of the section
     * @param type ViewSection constructor to be used
     * @returns Promise resolving to object specified by type
     */
    getSection<T extends ViewSection>(title: string, type: ViewSectionConstructor<T>): Promise<T>;
    /**
     * Retrieves a collapsible view content section by predicate.
     * Generic parameter allows caller to cast returned section to
     * desired type, however it is caller's responsibility to check
     * whether type is compatible with desired type.
     * @param predicate Predicate to be used when searching
     * @returns Promise resolving to ViewSection object
     */
    getSection<T extends ViewSection>(predicate: (section: ViewSection) => boolean | PromiseLike<boolean>): Promise<T>;
    /**
     * Retrieves a collapsible view content section by predicate.
     * Type parameter allows caller to use custom section implementation,
     * however it is caller's responsibility to check
     * whether type is compatible with desired type.
     * @param predicate Predicate to be used when searching
     * @param type ViewSection constructor to be used
     * @returns Promise resolving to object specified by type
     */
    getSection<T extends ViewSection>(predicate: (section: ViewSection) => boolean | PromiseLike<boolean>, type: ViewSectionConstructor<T>): Promise<T>;
    /**
     * Retrieves all the collapsible view content sections
     * @returns Promise resolving to array of ViewSection objects
     */
    getSections(): Promise<ViewSection[]>;
    private createSection;
}
